function y1=AEP_inattention

% This function is used to produce the numbers in table 1 of Abel, Eberly
% and Panageas (2013) "Optimal inattention to the stock market with
% information costs and transactions costs". 

% Taking a set of parameters the
% program reports a) whether the parameters are such that a time-dependent
% rule emerges in the long run and b) what is the associated inattention
% interval. 

% To use the function to reproduce the results of table 1, start with a
% level of \theta_2=0. In this special case (no fixed costs) there is
% always absorption in the long run. The resulting inattention interval is
% what is reported in the first column of the table.

% Moreover, the code reports the
% associated welfare cost using the formulas given in footnote
% 24 of the paper.

% In subsequent runs keep increasing \theta_2 in small increments until the message 
% "Theta_2 too big: No absorption" appears. This gives the critital \theta_2 for which 
% the time-dependent rule does not arise in the long run. 

% Column (3) in the paper reports the value of \theta_2 that is
% "marginally" (i.e. 10^-7 smaller)  than the critical \theta_2 that leads to no absorption, while column (4) 
% reports the associated inattention interval.


global rho; % Rate of time preference
global alpha; % Risk aversion

% Rates of return
global r_f; % Bond rate of return
global r_l; % Rate of return on the liquid asset
global mu; % Rate of return for risky stocks
global sigma; % Volatility of risky return

% Proportional and fixed costs
global kappa; % (Fixed) Observation cost;
global theta_2; % (Fixed) cost of transacting;
global psi_s; % Proportional selling cost;
global psi_b; % Proportional buying cost;

rho=0.01;%0.02
alpha=4;%3
r_f=0.02;%0.03
r_l=0.01;%0
mu=0.06;%0.07
sigma=0.16;%0.2
kappa=0.0001;%kappa=0.0001;
theta_2=0.0000065;
psi_s=0.01; % The inattention interval is not affected by the choices of \psi_s and \psi_b (see e.g. Proposition 6 in the appendix)
psi_b=0.01;
chi=(rho-(1-alpha)*r_l)/alpha; 
phi=min(1,1/alpha*(mu-r_f)/sigma^2); %approximate optimal portfolio. Allows the code to run faster without any significant effects on outcomes. To obtain exact portfolios, simply uncomment the relevant sections in the code that are marked as such.
toler=1.0e-6;

disp('The code takes about a minute to run on a Pentium i7 laptop')
adj_fac=1;

% First compute the inattention interval assuming theta_2=0
temp_theta=theta_2;
theta_2=0;
tau_sol_2_0=fzero(@ret_der,[10*kappa 2])
pi_and_v=addit_info(tau_sol_2_0);
welfare_cost=(1-(1-(1-alpha)*kappa/tau_sol_2_0*exp(rho))^(1/(1-alpha)))*pi_and_v(2)*(1-psi_s)/(1-psi_s+pi_and_v(2))*10^6;

% Restore the actual value of theta_2 and recompute the inattention
% interval
theta_2=temp_theta;
tau_sol=fzero(@ret_der,[kappa*10 2])
ih=addit_info(tau_sol);
v_pi_1=ih(1);
v_0=(1-theta_2)^(1-alpha)*((1-psi_s)/(ih(2)+1-psi_s))^(1-alpha)*v_pi_1;

wmega_vals=linspace(0.0000005,ih(2),40);
right_hand=(1-theta_2)^(1-alpha)*((1-psi_s+wmega_vals)./(1-psi_s+ih(2))).^(1-alpha)*v_pi_1;

tic
tau_wm=zeros(size(wmega_vals));
check=zeros(size(wmega_vals));
valfn=zeros(size(wmega_vals));
for i=1:length(wmega_vals)-1
    adj_fac=(ih(2)/(wmega_vals(i)))^(1-alpha);
    wme=wmega_vals(i);
    [tau_wm(i) check(i)]=fzero(@ret_der,[kappa/1000 tau_sol]);
    valfn(i)=addit_info(tau_wm(i));
end
tau_wm(i+1)=tau_sol;
valfn(i+1)=v_pi_1;


V_cand=min(right_hand,valfn);
options1 = optimset('Display','Off','GradObj','Off','TolCon',1e-8,'TolFun',1e-8,'TolX',1e-8,'UseParallel','Never', 'Algorithm','active-set');
wme=ih(2);

v_x=linspace(-8,8,20); %  
dx=max(diff(v_x));
            
            guess_c=ih(2)/tau_sol;
            constants=linspace(guess_c*0.8,guess_c*1.2,10);
            objvals=zeros(length(constants),20);
            port=1/(alpha)*(mu-r_f)/(sigma^2);

            for j=1:length(constants);lin(j,:)=linspace(kappa+0.000001,wme/constants(j),20);
            for n=1:length(lin(j,:));
                objvals(j,n)=ret_vals([constants(j)*lin(j,n);port;lin(j,n)]);
            end;
            end
            
            mm=min(min(objvals));
            
            [xxx,Ixx]=min(objvals); 
            [Dxx,Zxx]=min(xxx);
            
            x_guess(1,1)=constants(Ixx(Zxx))*lin(Ixx(Zxx),Zxx);
            x_guess(2,1)=1/alpha*(mu-r_f)/(sigma^2);
            x_guess(3,1)=lin(Ixx(Zxx),Zxx);
            
            
            disp(['For theta_2=0, the time between adjustments is ' num2str(tau_sol_2_0)])
            if mm<v_pi_1
                disp(['Theta_2=' num2str(theta_2) ' too big: No absorption'])
            else
                pol = fmincon(@ret_vals,x_guess,[1 0 0;-1 0 0;0 0 -1;0 1 0;0 -1 0],[wme;-0.00000001;-(kappa+0.01);1;0],[],[],[],[],[],options1);
                 if pol(1)==ih(2)
                    disp(['For theta_2=' num2str(theta_2) ', absorption obtains and the time between adjustments is ' num2str(tau_sol)]);
                    disp(['pi1 is equal to ' num2str(ih(2)) ' and the dollar equivalent cost is ' num2str(welfare_cost) ' dollars'])
                else
                    if v_pi_1<ret_vals(pol)
                        disp(['For theta_2=' num2str(theta_2) ', absorption obtains and the time between adjustments is ' num2str(tau_sol)]);
                        disp(['pi1 is equal to ' num2str(ih(2)) ' and the dollar equivalent cost is ' num2str(welfare_cost) ' dollars'])
                    else
                        disp(['Theta_2=' num2str(theta_2) ' too big: No absorption']);
                    end
                end
            end
                
 toc       
            

    function y=ret_vals(x)
        c=x(1);
        tau=x(3);
        phi=x(2);
        h=(1-exp(-chi*tau))/chi;
        out1=(1-kappa*(1-alpha)*exp(-rho*tau)/tau)*h^alpha*c^(1-alpha);
        v=@(x) interp1(wmega_vals,V_cand,max(0,real(x)),'linear','extrap');
        ret=@(x) phi*exp((mu-0.5*sigma^2)*tau+sigma*sqrt(tau)*x)+(1-phi)*exp(r_f*tau);
        np=@(x) exp(-rho*tau)*ret(x').^(1-alpha).*v(((wme-c)*exp(r_l*tau)./ret(x))').*normpdf(x',0,1);
        out2=quad(np,-8,8,toler);%sum(np(v_x))*dx; Replace with this command to make the code faster, without any significant impact on the results
        y=out1+out2;
    end

    function z=ret_der(tau)
        % This section of the code uses the first order conditions reported
        % in the first working paper version of Abel, Eberly, Panageas.
        
    phi=fminsearch(@focs,1/alpha*(mu-r_f)/sigma^2);% Comment this line for faster computation (uses an approximate expression for phi)
    ret = @(x) phi*exp((mu-0.5*sigma^2)*tau+sigma*sqrt(tau)*x)+(1-phi)*exp(r_f*tau);
    J1 = @(x) ret(x).^(1-alpha).*normpdf(x,0,1);
    J= (1-theta_2)^(1-alpha)*quad(J1,-8,8,toler)*exp(-rho*tau);
    pi_star=(1-psi_s)*(J^(-1/alpha)-1);
    h=(1-exp(-chi*tau))/chi;
    ret_der_m = @(x) ((mu-0.5*sigma^2)+sigma/2/sqrt(tau).*x).*phi.*exp((mu-0.5*sigma^2)*tau+sigma*sqrt(tau)*x)+r_f*(1-phi)*exp(r_f*tau);
    J_tau_1=@(x) ret_der_m(x).*ret(x).^(-alpha).*normpdf(x,0,1);
    J_tau=(1-alpha)*(1-theta_2)^(1-alpha)*exp(-rho*tau)*quad(J_tau_1,-8,8,toler)-rho*J;
    
    if adj_fac==1
    tt11=(rho+1/tau)*((1-alpha)*kappa*exp(-rho*(tau-1))/tau)/(1-(1-alpha)*kappa*exp(-rho*(tau-1))/tau);
    z=tt11+alpha*exp(-chi*tau)/h+J_tau/J*(1-psi_s)/pi_star;
    else
    tt11=(rho+1/tau)*((1-alpha)*kappa*exp(-rho*(tau-1))/tau)/(1-(1-alpha)*kappa*exp(-rho*(tau-1))/tau);
    z=(1-kappa*(1-alpha)*exp(-rho*tau)/tau)*h^alpha*wme^(1-alpha)*(tt11+alpha*exp(-chi*tau)/h)+J_tau/(1-theta_2)^(1-alpha)*v_0;   
    end
    
            function y=focs(ph)

                P=@(x) exp((mu-(sigma^2)/2)*tau+sigma*sqrt(tau)*x);
                R=@(x) (exp(r_f*tau)+ph*(P(x)-exp(r_f*tau)));
                f=@(x) exp(-(x.^2)/2)/(sqrt(2*pi));
                int3=@(x) ((R(x).^(-alpha)).*(P(x)-exp(r_f*tau))).*f(x);
                foc2=quad(int3,-8,8,toler);

                y=foc2^2;

            end
    
    
    end

    function z=addit_info(tau)
    phi=fminsearch(@focs,1/alpha*(mu-r_f)/sigma^2); 
    ret = @(x) phi*exp((mu-0.5*sigma^2)*tau+sigma*sqrt(tau)*x)+(1-phi)*exp(r_f*tau);
    J1 = @(x) ret(x).^(1-alpha).*normpdf(x,0,1);
    J= (1-theta_2)^(1-alpha)*quad(J1,-8,8,toler)*exp(-rho*tau);
    pi_star=(1-psi_s)*(J^(-1/alpha)-1);
    h=(1-exp(-chi*tau))/chi;
    if adj_fac==1
        z(1)=(1-kappa*(1-alpha)*exp(-rho*tau)/tau)*h^(alpha)*pi_star^(1-alpha)/(1-J*((1-psi_s)/(1-psi_s+pi_star))^(1-alpha));
        z(2)=pi_star;
    else
        z=(1-kappa*(1-alpha)*exp(-rho*tau)/tau)*h^(alpha)*wme^(1-alpha)+J/((1-theta_2)^(1-alpha))*v_0;
    end
    
            function y=focs(ph)

                P=@(x) exp((mu-(sigma^2)/2)*tau+sigma*sqrt(tau)*x);
                R=@(x) (exp(r_f*tau)+ph*(P(x)-exp(r_f*tau)));
                f=@(x) exp(-(x.^2)/2)/(sqrt(2*pi));
                int3=@(x) ((R(x).^(-alpha)).*(P(x)-exp(r_f*tau))).*f(x);
                foc2=quad(int3,-8,8,toler);

                y=foc2^2;

            end
    
    end

end
